package com.uas.erp.manage.server.api.v1;

import com.uas.erp.manage.server.domain.ClientInfo;
import com.uas.erp.manage.server.entity.Client;
import com.uas.erp.manage.server.service.ClientProjectService;
import com.uas.erp.manage.server.service.ClientContainerService;
import com.uas.erp.manage.server.service.ClientService;
import com.uas.erp.manage.server.util.IpUtils;
import com.uas.erp.manage.server.web.ResponseWrap;
import org.apache.catalina.servlet4preview.http.HttpServletRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.web.PageableDefault;
import org.springframework.http.ResponseEntity;
import org.springframework.util.Assert;
import org.springframework.web.bind.annotation.*;

import java.util.Date;

/**
 * Created by Pro1 on 2017/6/23.
 */
@RestController
@RequestMapping(path = "/v1/client")
public class ClientController {

    @Autowired
    private ClientService clientService;

    @Autowired
    private ClientProjectService clientProjectService;

    @Autowired
    private ClientContainerService clientContainerService;

    @GetMapping(path = "/list")
    public ResponseEntity getClients(@PageableDefault(size = 15, page = 0, sort = { "name" }) Pageable pageable) {
        return ResponseWrap.ok(clientService.getClients(pageable));
    }

    @GetMapping(path = "/{clientId}/projects")
    public ResponseEntity getClientProjects(@PathVariable("clientId") long clientId) {
        return ResponseWrap.ok(clientProjectService.findByClient(clientId));
    }

    @GetMapping(path = "/project/list")
    public ResponseEntity getClientProjects(@PageableDefault(size = 15, page = 0, sort = { "description" }) Pageable pageable, String description) {
        return ResponseWrap.ok(clientProjectService.findByDescription(pageable, description));
    }

    /**
     * 这个接口给客户端主动发起将信息记录到中心
     * @param request
     * @param info
     * @return
     */
    @PostMapping(path = "/info")
    public ResponseEntity saveClientInfo(HttpServletRequest request, @RequestBody ClientInfo info) {
        Assert.notNull(info.getClientId());
        Assert.notNull(info.getName());
        info.setIp(IpUtils.getIpAddr(request));
        info.setLastConnectTime(new Date().getTime());
        Client client = clientService.safetySave(info.getClient());
        clientProjectService.safetySave(info.getClientProjects(client));
        clientContainerService.safetySave(info.getClientContainers(client));
        return ResponseWrap.ok();
    }

}
