package com.uas.erp.manage.server.service;

import com.uas.erp.manage.server.entity.Client;
import com.uas.erp.manage.server.entity.ClientProject;
import com.uas.erp.manage.server.entity.ClientProjectDeployTask;
import com.uas.erp.manage.server.entity.ClientProjectDeployTaskItem;
import com.uas.erp.manage.server.repository.ClientProjectDeployTaskItemRepository;
import com.uas.erp.manage.server.repository.ClientProjectRepository;
import com.uas.erp.manage.server.web.ResultWrap;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.web.client.RestTemplate;

import javax.persistence.criteria.*;
import java.util.Date;
import java.util.List;

/**
 * Created by Pro1 on 2017/6/23.
 */
@Service
public class ClientProjectService {

    @Autowired
    private ClientService clientService;

    @Autowired
    private ClientProjectRepository clientProjectRepository;

    @Autowired
    private ClientProjectDeployTaskItemRepository clientProjectDeployTaskItemRepository;

    @Autowired
    private RestTemplate restTemplate;

    public List<ClientProject> findByClient(long clientId) {
        return clientProjectRepository.findByClientId(clientId);
    }

    public List<ClientProject> findByGroupIdAndArtifactId(String groupId, String artifactId) {
        return clientProjectRepository.findByGroupIdAndArtifactId(groupId, artifactId);
    }

    public List<ClientProject> findByClientIdAndGroupIdAndArtifactId(long clientId, String groupId, String artifactId) {
        return clientProjectRepository.findByClientIdAndGroupIdAndArtifactId(clientId, groupId, artifactId);
    }

    /**
     * 分页查询
     * @param pageable
     * @return
     */
    public Page<ClientProject> findByDescription(Pageable pageable, final String description) {
        return clientProjectRepository.findAll(new Specification<ClientProject>() {
            @Override
            public Predicate toPredicate(Root<ClientProject> root, CriteriaQuery<?> criteriaQuery, CriteriaBuilder criteriaBuilder) {
                Path<String> desc = root.get("description");
                return criteriaBuilder.like(desc, "%" + description + "%");
            }
        }, pageable);
    }

    public void safetySave(ClientProject project) {
        ClientProject oldOne = clientProjectRepository.findByClientIdAndProjectId(project.getClientId(), project.getProjectId());
        if (null != oldOne) {
            project.setId(oldOne.getId());
        }
        clientProjectRepository.save(project);
    }

    public void safetySave(List<ClientProject> projects) {
        if (!CollectionUtils.isEmpty(projects))
            for (ClientProject project:projects) {
                safetySave(project);
            }
    }

    @Async
    public void runDeployTask(ClientProject project, ClientProjectDeployTask task) {
        Client client = clientService.getClient(project.getClientId());
        ClientProjectDeployTaskItem taskItem = new ClientProjectDeployTaskItem(task, project);
        taskItem = clientProjectDeployTaskItemRepository.save(taskItem);
        taskItem.setStartTime(new Date().getTime());
        taskItem.setStatus(ClientProjectDeployTaskItem.Status.RUNNING.name());
        taskItem = clientProjectDeployTaskItemRepository.save(taskItem);
        try {
            ResultWrap<String> result = restTemplate.getForObject(client.getBasePath() + "/v1/project/{projectId}/deploy",
                    ResultWrap.class, project.getProjectId());
            if (result.isSuccess()) {
                taskItem.setSuccess(true);
            } else {
                taskItem.setMessage(result.getMessage());
            }
        } catch (Exception e) {
            taskItem.setSuccess(false);
            taskItem.setMessage(e.getMessage());
        } finally {
            taskItem.setStatus(ClientProjectDeployTaskItem.Status.FINISHED.name());
            taskItem.setEndTime(new Date().getTime());
            clientProjectDeployTaskItemRepository.save(taskItem);
        }
    }

}
